package de.tudresden.inf.st.eraser;

import beaver.Parser;
import de.tudresden.inf.st.eraser.deserializer.ASTNodeDeserializer;
import de.tudresden.inf.st.eraser.jastadd.model.MqttHandler;
import de.tudresden.inf.st.eraser.jastadd.model.MqttRoot;
import de.tudresden.inf.st.eraser.jastadd.model.NumberItem;
import de.tudresden.inf.st.eraser.jastadd.model.Root;
import de.tudresden.inf.st.eraser.openhab2.OpenHab2Importer;
import de.tudresden.inf.st.eraser.util.ParserUtils;
import de.tudresden.inf.st.eraser.util.TestUtils;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.util.concurrent.CountDownLatch;

/**
 * Main entry point for testing eraser.
 * @author rschoene - Initial contribution
 */
@SuppressWarnings({"unused", "RedundantThrows", "CommentedOutCode"})
public class Main {

  public static void main(String[] args) throws Exception {
//    testSimple();
//    Root model = testParser();
//    Root model = importFromOpenHab();
//    testPrinterWith(model);
    testRagConnectConnection();
  }

  private static void testRagConnectConnection() throws IOException, InterruptedException {
    TestUtils.ModelAndItem mai = TestUtils.createModelAndItem(0, true);
    NumberItem item = mai.item;
    MqttRoot mqttRoot = mai.model.getRoot().getMqttRoot();
    mqttRoot.getHost().setHostName("localhost");
    MqttHandler handler = new MqttHandler().dontSendWelcomeMessage().setHost("localhost");
    mqttRoot.setIncomingPrefix("inc");
    mqttRoot.setOutgoingPrefix("out");
    mqttRoot.ensureCorrectPrefixes();

    CountDownLatch exit = new CountDownLatch(1);

//    item.connect_state(mqttUri(mqttRoot.getIncomingPrefix() + item.getID()));
//    item.connect_state(mqttUri(mqttRoot.getOutgoingPrefix() + item.getID()), true);

    boolean success = mai.model.connectAllItems();
    System.out.println("success = " + success);

    handler.newConnection("model", bytes -> System.out.println(mai.model.prettyPrint()));
    handler.newConnection("exit", bytes -> exit.countDown());

    exit.await();
    handler.close();
    mai.model.getRoot().ragconnectCloseConnections();
  }

  private static String mqttUri(String s) {
    return "mqtt://localhost/" + s;
  }

  private static double readFromSystemIn(BufferedReader in, String prompt) throws IOException {
    System.out.println(prompt);
    String line = in.readLine();
    return Double.parseDouble(line);
  }

  private static Root testParser() throws IOException, Parser.Exception {
//    return ParserUtils.load("openhab-data.eraser", Main.class);
    ParserUtils.setVerboseLoading(true);
    return ParserUtils.load("easy-data.eraser", Main.class);
  }

  private static void testPrinterWith(Root model) {
    model.flushTreeCache();
    System.out.println("Got model: " + model.getSmartHomeEntityModel().description());
    System.out.println("PrettyPrinted:");
    System.out.println(model.prettyPrint());
  }

  private static void testSimple() {
    String s = "hingType: id=";
    for (Character c : s.toCharArray()) {
      System.out.println("'" + c + "'.isJavaIdentifierPart: " + Character.isJavaIdentifierPart(c));
    }
    System.exit(0);
  }

  public static Root importFromOpenHab() {
    OpenHab2Importer importer = new OpenHab2Importer();
    return importer.importFrom("192.168.1.250", 8080).getRoot();
  }

  public static Root importFromFile() {
    final String filename = "/openhab2-data.json";
    InputStream inputStream = Main.class.getResourceAsStream(filename);
    return ASTNodeDeserializer.read(inputStream);
  }
}
