package de.tudresden.inf.st.eraser;

import de.tudresden.inf.st.eraser.jastadd.model.*;
import de.tudresden.inf.st.eraser.util.ParserUtils;
import de.tudresden.inf.st.eraser.util.TestUtils;
import de.tudresden.inf.st.eraser.util.TestUtils.ModelAndItem;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.After;
import org.junit.Before;
import org.junit.jupiter.api.*;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.time.Instant;
import java.util.concurrent.TimeUnit;

import static de.tudresden.inf.st.eraser.util.TestUtils.getMqttHost;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

/**
 * Test for the combination of MQTT and rules.
 *
 * @author rschoene - Initial contribution
 */
@Tag("mqtt")
@Tag("New")
public class MqttRulesTest {

  private static final Logger logger = LogManager.getLogger(MqttRulesTest.class);

  private static final String topic = "a/b";
  private MqttHandler handler;

  @BeforeEach
  public void activateHandler() throws IOException {
    handler = new MqttHandler().dontSendWelcomeMessage().setHost(getMqttHost());
    assertTrue(handler.waitUntilReady(2, TimeUnit.SECONDS), "Could not connect to MQTT broker");
    handler.publish("test", ("before at " + Instant.now()).getBytes(StandardCharsets.UTF_8));
  }

  @AfterEach
  public void deactivateHandler() {
    if (handler != null) {
      handler.publish("test", ("after at " + Instant.now()).getBytes(StandardCharsets.UTF_8));
      handler.close();
    }
  }

  @Test
  public void receiveItemTriggerRule(TestInfo testInfo) throws Exception {
    String displayName = testInfo.getDisplayName();
    handler.publish("test", ("starting " + displayName).getBytes(StandardCharsets.UTF_8));
    // Given model with two items, one to receive updates from mqtt and another to be updated
    ModelAndItem mai = MqttTests.createModelAndSetupMqttRoot();
    Root root = mai.model.getRoot();
    String incomingPrefix = root.getMqttRoot().getIncomingPrefix();
    String usedTopic = incomingPrefix + topic;

    NumberItem receivingItem = mai.item;
    receivingItem.setTopicString(topic);
    NumberItem itemToBeUpdated = TestUtils.addItemTo(mai.model, 2.0, true);
    assertTrue(mai.model.connectAllItems(), "Could not connect items");
    TestUtils.waitForMqtt();

    // ... and a rule is defined for item to be updated
    Rule rule = new Rule();
    SetStateFromExpression action = new SetStateFromExpression();
    // TODO item1 should be referred to as triggering item
    action.setNumberExpression(ParserUtils.parseNumberExpression("(" + receivingItem.getID() + " + 7)", root));
    action.setAffectedItem(itemToBeUpdated);
    rule.addAction(action);
    RulesTest.CountingAction counter = new RulesTest.CountingAction();
    rule.addAction(counter);
//    root.ragconnectJavaRegisterConsumer(itemToBeUpdated.getID(), bytes -> {
//      String state = new String(bytes, StandardCharsets.UTF_8);
//      itemToBeUpdated.stateUpdated(itemToBeUpdated.sendState);
//    });

    root.addRule(rule);
    rule.activateFor(receivingItem);

//    // try manually
//    logger.warn(counter.counters);
//    receivingItem.triggerStateUpdated();
//    logger.warn(counter.counters);
//    receivingItem.set_state(3.0);
//    receivingItem.triggerStateUpdated();
//    logger.warn(counter.counters);
//    receivingItem.set_state(9.0);
//    logger.warn(counter.counters);

    // When a message is published on the topic the first item is receiving
//    itemToBeUpdated.dumpDependencies();
    logger.debug("before '{}' + {}", receivingItem.getState(), root.ragconnectEvaluationCounterSummary());
    handler.publish(usedTopic, "4.0".getBytes(StandardCharsets.UTF_8));
    TestUtils.waitForMqtt();
    logger.debug("after '{}' + {}", receivingItem.getState(), root.ragconnectEvaluationCounterSummary());
//    itemToBeUpdated.dumpDependencies();

    // Then the state of both items should be updated
    logger.warn(counter.counters);
    // v-- this needs to be called automatically : TODO
//    receivingItem.triggerStateUpdated();
//    logger.warn(counter.counters);

    assertEquals(4.0, receivingItem.getState(), TestUtils.DELTA);
    assertEquals(11.0, itemToBeUpdated.getState(), TestUtils.DELTA);
  }

}
